==> kaqing-0.28.0-py3-none-any.whl: walker/checks/memory.py <==
from walker.checks.check import Check
from walker.checks.check_context import CheckContext
from walker.checks.check_result import CheckResult
from walker.checks.issue import Issue
from walker.k8s_utils import cassandra_pod_exec, get_container, get_metrics

class Memory(Check):
    def name(self):
        return 'memory'

    def check(self, ctx: CheckContext) -> CheckResult:
        issues: list[Issue] = []
        details = {
            'name': ctx.pod,
            'namespace': ctx.namespace,
            'statefulset': ctx.statefulset,
            'used': 'NA',
            'request': 'NA',
            'limit': 'NA'
        }

        try:
            metrics = get_metrics(ctx.namespace, ctx.pod, container_name='cassandra')
            details['used'] = metrics['usage']['memory']

            container = get_container(ctx.namespace, ctx.pod, container_name='cassandra')
            if container.resources.requests and "memory" in container.resources.requests:
                details['request'] = container.resources.requests["memory"]
            if container.resources.limits and "memory" in container.resources.limits:
                details['limit'] = container.resources.limits["memory"]

            if issue := self.find_error(ctx, 'Not marking nodes down due to local pause',
                                        'local pause due to memory pressure'):
                issues.append(issue)
            if issue := self.find_error(ctx, 'java.lang.OutOfMemoryError: Direct buffer memory',
                                        'direct buffer OOM'):
                issues.append(issue)
            if issue := self.find_error(ctx, 'query aborted (see tombstone_failure_threshold)',
                                        'too many tombstones'):
                issues.append(issue)
        except Exception as e:
            issues.append(self.issue_from_err(statefulset_name=ctx.statefulset, ns=ctx.namespace, pod_name=ctx.pod, exception=e))

        return CheckResult(self.name(), details, issues)

    def find_error(self, ctx: CheckContext, pattern: str, issue_desc: str):
        escaped = pattern.replace('"', '\"')
        result = cassandra_pod_exec(ctx.pod, ctx.namespace, f'tac /c3/cassandra/logs/system.log | grep "{escaped}" | head -1', show_out=ctx.show_output)
        if result.stdout.find(pattern) > 0:
            return Issue(
                statefulset=ctx.statefulset,
                namespace=ctx.namespace,
                pod=ctx.pod,
                category=self.name(),
                desc=f"node: {ctx.host_id} reported {issue_desc}",
                details=result.stdout.strip(' \r\n'),
                # qing admin restart -n gkeops845 -p cs-d0767a536f-cs-d0767a536f-default-sts-0
                suggestion=f"qing admin restart {ctx.pod}@{ctx.namespace}"
            )

        return None

    def help(self):
        return f'{Memory().name()}: check current container memory usage and scan system log for an error'

==> kaqing-0.34.0-py3-none-any.whl: walker/checks/memory.py <==
from walker.checks.check import Check
from walker.checks.check_context import CheckContext
from walker.checks.check_result import CheckResult
from walker.checks.issue import Issue
from walker.k8s_utils import cassandra_pod_exec, get_container, get_metrics

class Memory(Check):
    def name(self):
        return 'memory'

    def check(self, ctx: CheckContext) -> CheckResult:
        issues: list[Issue] = []
        details = {
            'name': ctx.pod,
            'namespace': ctx.namespace,
            'statefulset': ctx.statefulset,
            'used': 'NA',
            'request': 'NA',
            'limit': 'NA'
        }

        try:
            metrics = get_metrics(ctx.namespace, ctx.pod, container_name='cassandra')
            details['used'] = metrics['usage']['memory']

            container = get_container(ctx.namespace, ctx.pod, container_name='cassandra')
            if container.resources.requests and "memory" in container.resources.requests:
                details['request'] = container.resources.requests["memory"]
            if container.resources.limits and "memory" in container.resources.limits:
                details['limit'] = container.resources.limits["memory"]

            if issue := self.find_error(ctx, 'Not marking nodes down due to local pause',
                                        'local pause due to memory pressure'):
                issues.append(issue)
            if issue := self.find_error(ctx, 'java.lang.OutOfMemoryError: Direct buffer memory',
                                        'direct buffer OOM'):
                issues.append(issue)
            if issue := self.find_error(ctx, 'query aborted (see tombstone_failure_threshold)',
                                        'too many tombstones'):
                issues.append(issue)
        except Exception as e:
            issues.append(self.issue_from_err(statefulset_name=ctx.statefulset, ns=ctx.namespace, pod_name=ctx.pod, exception=e))

        return CheckResult(self.name(), details, issues)

    def find_error(self, ctx: CheckContext, pattern: str, issue_desc: str):
        escaped = pattern.replace('"', '\"')
        result = cassandra_pod_exec(ctx.pod, ctx.namespace, f'tac /c3/cassandra/logs/system.log | grep "{escaped}" | head -1', show_out=ctx.show_output)
        if result.stdout.find(pattern) > 0:
            return Issue(
                statefulset=ctx.statefulset,
                namespace=ctx.namespace,
                pod=ctx.pod,
                category=self.name(),
                desc=f"node: {ctx.host_id} reported {issue_desc}",
                details=result.stdout.strip(' \r\n'),
                # qing admin restart -n gkeops845 -p cs-d0767a536f-cs-d0767a536f-default-sts-0
                suggestion=f"qing restart {ctx.pod}@{ctx.namespace}"
            )

        return None

    def help(self):
        return f'{Memory().name()}: check current container memory usage and scan system log for an error'
